var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
// deno-lint-ignore-file ban-ts-comment
import { Layout } from '../layout/index.js';
import { BackgroundColorStyle, BorderColorStyle, BorderRadiusStyle, BorderWidthStyle, ColorStyle, FontSizeStyle, ZIndexStyle, OpacityStyle, FontStyleStyle, TextAlignStyle } from './properties.js';
/**
 * A function that decorates properties on the style object.
 */
export function style(config) {
    return function (_target, property) {
        if (config.defaultValue !== undefined) {
            _target._nativeStyleDefaults.set(property, config.defaultValue);
        }
        Object.defineProperty(_target, property, {
            get() {
                if (config.getNative && config.converter?.fromNative) {
                    return config.converter.fromNative(property, config.getNative(this));
                }
                return this.get(property);
            },
            set(value) {
                const oldValue = this.get(property);
                const hasChanged = !config.hasChanged ? oldValue !== value : config.hasChanged?.(oldValue, value);
                if (!hasChanged)
                    return;
                this.set(property, value);
                const nativeValue = config.converter?.toNative?.(property, value) ?? value;
                const SetNativeKey = `${property}SetNative`;
                const pendingSetNative = function () {
                    if (this.node[SetNativeKey]) {
                        this.node[SetNativeKey].call(this.node, property, value, config);
                    }
                    else {
                        config.setNative(this, property, nativeValue, config);
                    }
                    if (config.shouldLayout) {
                        if (this.node.isLeaf) {
                            this.node.yogaNode.markDirty();
                        }
                        Layout.computeAndLayout(this.node);
                    }
                };
                if (!this.node.nativeView) {
                    this.pendingSetNative.set(property, pendingSetNative.bind(this));
                    return;
                }
                else {
                    pendingSetNative.call(this);
                }
            },
        });
    };
}
export function flex() {
    return style({
        setNative: (style, key, value) => {
            //@ts-ignore
            if (!Layout.Setters[key]) {
                console.error(`Flex property ${key} is not supported`);
                return;
            }
            if (style.node) {
                //@ts-ignore
                Layout.Setters[key](style.node.yogaNode, value);
                Layout.computeAndLayout(style.node);
            }
        },
        hasChanged: (oldValue, newValue) => oldValue !== newValue,
        converter: {
            toNative: (key, value) => Layout.ConvertToYogaValue[key](value),
        },
    });
}
export class Style extends Map {
    constructor(node) {
        super();
        this.node = node;
        this._nativeStyleDefaults = new Map();
        this.pendingSetNative = new Map();
    }
    set(key, value) {
        super.set(key, value);
        return this;
    }
    get(key) {
        return super.get(key);
    }
    clear() {
        for (const key of this.keys()) {
            //@ts-ignore
            this[key] = undefined;
        }
        super.clear();
    }
}
/**
 * Install a new style property.
 */
Style.install = style;
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "alignContent", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "alignItems", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "alignSelf", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "aspectRatio", void 0);
__decorate([
    style(BorderWidthStyle),
    __metadata("design:type", Object)
], Style.prototype, "borderWidth", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "direction", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "display", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "flex", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "flexBasis", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "flexDirection", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "flexGrow", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "flexShrink", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "flexWrap", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "width", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "height", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "justifyContent", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "gap", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "rowGap", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "columnGap", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "marginLeft", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "marginRight", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "marginTop", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "marginBottom", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "maxHeight", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "maxWidth", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "minHeight", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "minWidth", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "overflow", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "paddingLeft", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "paddingRight", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "paddingTop", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "paddingBottom", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "padding", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "top", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "left", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "right", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "bottom", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "end", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "start", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "marginBlock", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "marginBlockEnd", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "marginBlockStart", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "paddingBlock", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "paddingBlockEnd", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "paddingBlockStart", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "paddingHorizontal", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "paddingVertical", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "marginVertical", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "marginHorizontal", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "marginStart", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "marginEnd", void 0);
__decorate([
    flex(),
    __metadata("design:type", Object)
], Style.prototype, "paddingStart", void 0);
__decorate([
    style(BackgroundColorStyle),
    __metadata("design:type", Object)
], Style.prototype, "backgroundColor", void 0);
__decorate([
    style(ZIndexStyle),
    __metadata("design:type", Object)
], Style.prototype, "zIndex", void 0);
__decorate([
    style(FontSizeStyle),
    __metadata("design:type", Object)
], Style.prototype, "fontSize", void 0);
__decorate([
    style(FontStyleStyle),
    __metadata("design:type", Object)
], Style.prototype, "fontStyle", void 0);
__decorate([
    style(ColorStyle),
    __metadata("design:type", Object)
], Style.prototype, "color", void 0);
__decorate([
    style(BorderRadiusStyle),
    __metadata("design:type", Object)
], Style.prototype, "borderRadius", void 0);
__decorate([
    style(BorderColorStyle),
    __metadata("design:type", Object)
], Style.prototype, "borderColor", void 0);
__decorate([
    style(OpacityStyle),
    __metadata("design:type", Object)
], Style.prototype, "opacity", void 0);
__decorate([
    style(TextAlignStyle),
    __metadata("design:type", Object)
], Style.prototype, "textAlign", void 0);
//# sourceMappingURL=index.js.map